@@ -72,12 +72,18 @@ ZrtpQueue::~ZrtpQueue() {
 }
 
 int32_t
-ZrtpQueue::initialize(const char *zidFilename, bool autoEnable)
+ZrtpQueue::initialize(const char *zidFilename, bool autoEnable,
+                     ZrtpConfigure* config)
 {
     int32_t ret = 1;
 
     synchEnter();
 
+    ZrtpConfigure* configOwn = NULL;
+    if (config == NULL) {
+        config = configOwn = new ZrtpConfigure();
+        config->setStandardConfig();
+    }
     enableZrtp = autoEnable;
 
     if (staticTimeoutProvider == NULL) {
@@ -91,7 +97,7 @@ ZrtpQueue::initialize(const char *zidFilename, bool autoEnable)
             char *home = getenv("HOME");
             std::string baseDir = (home != NULL) ? (std::string(home) + std::string("/."))
                                                     : std::string(".");
-            fname = baseDir + std::string("GNUccRTP.zid");
+            fname = baseDir + std::string("GNUZRTP.zid");
             zidFilename = fname.c_str();
         }
         if (zf->open((char *)zidFilename) < 0) {
@@ -101,7 +107,10 @@ ZrtpQueue::initialize(const char *zidFilename, bool autoEnable)
     }
     if (ret > 0) {
         const uint8_t* ownZid = zf->getZid();
-        zrtpEngine = new ZRtp((uint8_t*)ownZid, (ZrtpCallback*)this, clientIdString);
+        zrtpEngine = new ZRtp((uint8_t*)ownZid, (ZrtpCallback*)this, clientIdString, config);
+    }
+    if (configOwn != NULL) {
+        delete configOwn;
     }
     synchLeave();
     return ret;
@@ -142,22 +151,13 @@ ZrtpQueue::takeInDataPacket(void)
     IncomingZRTPPkt* packet = NULL;
     // check if this could be a real RTP/SRTP packet.
     if ((*buffer & 0xf0) != 0x10) {
-
-        //  Could be real RTP, build a packet.
-        IncomingRTPPkt* pkt = new IncomingRTPPkt(buffer,rtn);
-
-        // Generic header validity check. If valid perform standard RTP handling
-        if (pkt->isHeaderValid()) {
-            return (rtpDataPacket(pkt, rtn, network_address, transport_port));
-        }
-        delete pkt;
-        return 0;
+        return (rtpDataPacket(buffer, rtn, network_address, transport_port));
     }
 
     // We assume all other packets are ZRTP packets here. Process
     // if ZRTP processing is enabled. Because valid RTP packets are
     // already handled we delete any packets here after processing.
-    if (enableZrtp) {
+    if (enableZrtp && zrtpEngine != NULL) {
         // Get CRC value into crc (see above how to compute the offset)
         uint16_t temp = rtn - CRC_SIZE;
         uint32_t crc = *(uint32_t*)(buffer + temp);
@@ -165,7 +165,8 @@ ZrtpQueue::takeInDataPacket(void)
 
         if (!zrtpCheckCksum(buffer, temp, crc)) {
             delete buffer;
-            zrtpUserCallback->showMessage(Warning, WarningCRCmismatch);
+            if (zrtpUserCallback != NULL)
+                zrtpUserCallback->showMessage(Warning, WarningCRCmismatch);
             return 0;
         }
 
@@ -198,10 +199,29 @@ ZrtpQueue::takeInDataPacket(void)
 }
 
 size_t
-ZrtpQueue::rtpDataPacket(IncomingRTPPkt* packet, int32 rtn, 
+ZrtpQueue::rtpDataPacket(unsigned char* buffer, int32 rtn, 
                          InetHostAddress network_address, 
                          tpport_t transport_port)
 {
+     // Special handling of padding to take care of encrypted content.
+    // In case of SRTP the padding length field is also encrypted, thus
+    // it gives a wrong length. Check and clear padding bit before
+    // creating the RTPPacket. Will be set and re-computed after a possible
+    // SRTP decryption.
+    uint8 padSet = (*buffer & 0x20);
+    if (padSet) {
+        *buffer = *buffer & ~0x20;          // clear padding bit
+    }    
+    //  build a packet. It will link itself to its source
+    IncomingRTPPkt* packet =
+        new IncomingRTPPkt(buffer,rtn);
+
+    // Generic header validity check.
+    if ( !packet->isHeaderValid() ) {
+        delete packet;
+        return 0;
+    }
+
     // Look for a CryptoContext for this packet's SSRC
     CryptoContext* pcc = getInQueueCryptoContext(packet->getSSRC());
 
@@ -240,7 +260,9 @@ ZrtpQueue::rtpDataPacket(IncomingRTPPkt* packet, int32 rtn,
         delete packet;
         return 0;
     }
-
+    if (padSet) {
+        packet->reComputePayLength(true);
+    }
     // get time of arrival
     struct timeval recvtime;
     gettimeofday(&recvtime,NULL);
@@ -644,6 +666,12 @@ bool ZrtpQueue::isMultiStream()  {
     return false;
 }
 
+bool ZrtpQueue::isMultiStreamAvailable()  {
+    if (zrtpEngine != NULL)
+        return zrtpEngine->isMultiStreamAvailable();
+    return false;
+}
+
 void ZrtpQueue::acceptEnrollment(bool accepted) {
     if (zrtpEngine != NULL)
         zrtpEngine->acceptEnrollment(accepted);
