@@ -27,11 +27,6 @@
  *
  * The ZID file holds information about peers.
  *
- * <p/>
- *
- * NOTE: This class is a friend of the ZIDRecord class. Please keep both
- * classes synchronized.
- *
  * @author: Werner Dittmann <Werner.Dittmann@t-online.de>
  */
 
@@ -47,6 +42,8 @@ private:
      */
     ZIDFile(): zidFile(NULL) {};
     ~ZIDFile();
+    void createZIDFile(char* name);
+    void checkDoMigration(char* name);
 
 public:
 
@@ -55,7 +52,7 @@ public:
      *
      * This method just creates an instance an store a pointer to it
      * in a static variable. The ZIDFile is a singleton, thus only
-     * <e>one</e> ZID file can be open at one time.
+     * <em>one</em> ZID file can be open at one time.
      *
      * @return
      *    A pointer to the global ZIDFile singleton instance.
@@ -66,11 +63,9 @@ public:
      *
      * This static function either opens an existing ZID file or
      * creates a new ZID file with the given name. The ZIDFile is a
-     * singleton, thus only <e>one</e> ZID file can be open at one
+     * singleton, thus only <em>one</em> ZID file can be open at one
      * time.
      *
-     * <p/>
-     *
      * To open another ZID file you must close the active ZID file
      * first.
      *
@@ -86,9 +81,9 @@ public:
      * Check if ZIDFile has an active (open) file.
      *
      * @return
-     *    1 if ZIDFile has an active file, 0 otherwise
+     *    True if ZIDFile has an active file, false otherwise
      */
-    int isOpen() { return (zidFile == NULL); };
+    bool isOpen() { return (zidFile != NULL); };
 
      /**
      * Close the ZID file.
@@ -100,17 +95,19 @@ public:
      * Get a ZID record from the active ZID file.
      *
      * The method get the identifier data from the ZID record parameter,
-     * locates the record in the ZID file and fills in the RS1 and RS2
-     * data. If no matching record exists in the ZID file the method creates
+     * locates the record in the ZID file and fills in the RS1, RS2, and
+     * other data. 
+     *
+     * If no matching record exists in the ZID file the method creates
      * it and fills it with default values.
      *
      * @param zidRecord
      *    The ZID record that contains the identifier data. The method
-     *    fills in the RS1 and RS2 data.
+     *    fills in data .
      * @return
-     *    TODO
+     *    Currently always 1 to indicate sucess
      */
-    unsigned int getRecord(ZIDRecord *zidRecord);
+    unsigned int getRecord(ZIDRecord* zidRecord);
 
     /**
      * Save a ZID record into the active ZID file.
@@ -122,7 +119,7 @@ public:
      * @param zidRecord
      *    The ZID record to save.
      * @return
-     *    1 on success, 0 on failure
+     *    1 on success
      */
     unsigned int saveRecord(ZIDRecord *zidRecord);
 
